@@ -12,36 +12,37 @@ from python_fixturify_project.exceptions import InvalidProjectError
 from python_fixturify_project.path_utils import create_directory, write_to_file
 from python_fixturify_project.utils import deep_merge, keys_exists
 
+DEFAULT_IGNORE_PATTERNS = ["**/.git", "**/.git/**"]
+
 
 class Project:
-    def __init__(self, base_dir=None, files=None):
-        self._base_dir = base_dir
+    def __init__(self, files=None, ignore_patterns=[]):
+        self._base_dir = ""
         self._files = files or {}
+        self._ignore_patterns = DEFAULT_IGNORE_PATTERNS + ignore_patterns
 
         self.write(self._files)
 
     def __del__(self):
         # Ensure we clean up the temp dir structure on delete
-        self.__exit__()
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, *args):
-        try:
-            shutil.rmtree(self.base_dir)
-        except FileNotFoundError:
-            # No need to do anything, file structure has already been cleaned up!
-            pass
+        self.dispose()
 
     @property
     def base_dir(self):
         """Gets the base directory path, usually a tmp directory unless a baseDir has been explicitly set."""
+        if self._base_dir == "":
+            raise Exception(
+                "Project has no base_dir yet. Either set one manually or call write to have one chosen for you."
+            )
+
         return self._base_dir
 
     @base_dir.setter
     def base_dir(self, value):
         """Sets the base directory of the project."""
+        if self._base_dir != "":
+            raise Exception("Project already has a base_dir")
+
         self._base_dir = value
 
     @base_dir.deleter
@@ -67,10 +68,12 @@ class Project:
         """Deletes the files corresponding to the project's directory structure."""
         del self._files
 
-    def __auto_base_dir(self):
-        """Creates and sets the base_dir if not explicitly configured during init"""
-        if not self.base_dir:
-            self.base_dir = tempfile.mkdtemp()
+    def dispose(self):
+        try:
+            shutil.rmtree(self._base_dir)
+        except FileNotFoundError:
+            # No need to do anything, file structure has already been cleaned up!
+            pass
 
     def merge_files(self, dir_json):
         """Merges an object containing a directory represention with the existing files."""
@@ -80,14 +83,15 @@ class Project:
         """Writes the existing files property containing a directory representation to the tmp directory."""
         if dir_json:
             self.merge_files(dir_json)
+
         self.__write_project()
 
-    def read(self, ignore_patterns=["**/.git", "**/.git/**"]):
+    def read(self):
         """Reads the contents of the base_dir to a dict and ignores any files/dirs matched by the glob expressions"""
         files: Dict[str, Any] = {}
 
         for path in Path(self.base_dir).rglob(
-            "*", exclude=ignore_patterns, flags=DOTGLOB | GLOBSTAR
+            "*", exclude=self._ignore_patterns, flags=DOTGLOB | GLOBSTAR
         ):
             rel_path = path.relative_to(self.base_dir)
 
@@ -104,12 +108,17 @@ class Project:
 
         return files
 
-    def get(self, object_path: str):
+    def get(self, object_path):
         if self._files == {}:
             self._files = self.read()
 
         return extract_dict(self._files, object_path)
 
+    def __auto_base_dir(self):
+        """Creates and sets the base_dir if not explicitly configured during init"""
+        if not self._base_dir:
+            self._base_dir = tempfile.mkdtemp()
+
     def __add_file(self, files, path, contents):
         file = os.path.basename(path)
         dir_name = os.path.dirname(path)
@@ -130,29 +139,30 @@ class Project:
         self.__auto_base_dir()
         self.__write(self.files, self.base_dir)
 
-    def __write(self, dir_structure, full_path):
+    def __write(self, files, full_path):
         """Recursive write helper function. Does the bulk of the work in terms of writing the directory structure"""
-        # Base case
-        if not dir_structure or not isinstance(dir_structure, dict):
+        if not files or not isinstance(files, dict):
             return
 
-        # Save the original path
         original_dir = Path(full_path)
-        for entry in dir_structure:
+
+        for entry in files:
             full_path = Path(full_path, entry)
+
             if not isinstance(entry, str) or entry == "":
                 raise InvalidProjectError(
                     "Invalid directory structure given. Each key must be a non-empty string"
                 )
-            if isinstance(dir_structure[entry], str):  # This is a file
-                write_to_file(full_path, dir_structure[entry])
+
+            if isinstance(files[entry], str):
+                write_to_file(full_path, files[entry])
             else:
                 if entry == "." or entry == "..":
                     raise InvalidProjectError('Directory entry must not be "." or ".."')
 
                 create_directory(full_path)
                 # Our recursion step, which should only happen if we find ourselves a nested directory
-                self.__write(dir_structure=dir_structure[entry], full_path=full_path)
+                self.__write(files=files[entry], full_path=full_path)
 
             # Reset the original path because Python will still remember the value of `full_path` even after we return from recursion
             full_path = original_dir
